#include <QtGui>
#include <QtXml>

#include "domitem.h"
#include "dommodel.h"

DomModel::DomModel(QDomDocument document, QObject *parent)
    : QAbstractItemModel(parent), domDocument(document)
{
    rootItem = new DomItem(domDocument, 0);
}

DomModel::~DomModel()
{
    delete rootItem;
}

int DomModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent);
    return 3;
}

QVariant DomModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    if (role != Qt::DisplayRole && role != RoleIdd)
        return QVariant();

    DomItem *item = static_cast<DomItem*>(index.internalPointer());

    if (role == RoleIdd) return item->Idd;

    QDomNode node = item->node();
    QStringList attributes;
    QDomNamedNodeMap attributeMap = node.attributes();

    switch (index.column())
    {
        case 0: return node.nodeName();

        case 1: for (int i = 0; i < attributeMap.count(); ++i)
                {
                    QDomNode attribute = attributeMap.item(i);
                    attributes << attribute.nodeName() + "=\""
                              +attribute.nodeValue() + "\"";
                }
                return attributes.join(" ");

        case 2: return node.nodeValue().split("\n").join(" ");

       default: return QVariant();
    }
}

Qt::ItemFlags DomModel::flags(const QModelIndex &index) const
{
    if (!index.isValid())
        return 0;

    return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
}

QVariant DomModel::headerData(int section, Qt::Orientation orientation,
                              int role) const
{
    if (orientation == Qt::Horizontal && role == Qt::DisplayRole)
    {
        switch (section)
        {
            case 0: return tr("Name");
            case 1: return tr("Attributes");
            case 2: return tr("Value");
           default: return QVariant();
        }
    }

    return QVariant();
}

QModelIndex DomModel::index(int row, int column, const QModelIndex &parent)
            const
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();

    DomItem *parentItem;

    if (!parent.isValid())
        parentItem = rootItem;
    else
        parentItem = static_cast<DomItem*>(parent.internalPointer());

    DomItem *childItem = parentItem->child(row);
    if (childItem)
        return createIndex(row, column, childItem);
    else
        return QModelIndex();
}

QModelIndex DomModel::parent(const QModelIndex &child) const
{
    if (!child.isValid())
        return QModelIndex();

    DomItem *childItem = static_cast<DomItem*>(child.internalPointer());
    DomItem *parentItem = childItem->parent();

    if (!parentItem || parentItem == rootItem)
        return QModelIndex();

    return createIndex(parentItem->row(), 0, parentItem);
}

int DomModel::rowCount(const QModelIndex &parent) const
{
    if (parent.column() > 0)
        return 0;

    DomItem *parentItem;

    if (!parent.isValid())
        parentItem = rootItem;
    else
        parentItem = static_cast<DomItem*>(parent.internalPointer());

    return parentItem->node().childNodes().count();
}

QModelIndex DomModel::findIDD(int idd)
{
    QModelIndexList idlist = match( index(0, 0, QModelIndex()), RoleIdd, QVariant(idd), 1,
                             Qt::MatchExactly | Qt::MatchRecursive | Qt::MatchWrap);
    QModelIndex idx = idlist.front();

    if (idx.isValid()) return idx;
    return QModelIndex();
}

DomItem *DomModel::getItem(const QModelIndex &index) const
{
    if (index.isValid())
    {
        DomItem *item = static_cast<DomItem*>(index.internalPointer());
        if (item) return item;
    }
    return rootItem;
}

bool DomModel::swapRows(int pos1, int pos2, const QModelIndex &parent)
{
    bool success = false;
    DomItem *parentItem = getItem(parent);

//    beginRemoveRows(parent, pos1, pos2);
    success = parentItem->swapChildrens(pos1, pos2);
//    endRemoveRows();
    return success;
}

bool DomModel::insertRows(int position, int rows, const QModelIndex &parent)
{
    bool success = false;
    DomItem *parentItem = getItem(parent);

    beginInsertRows(parent, position, position + rows - 1);
//    success = parentItem->insertChildrens(position, rows);
    endInsertRows();
    return success;
}

bool DomModel::removeRows(int position, int rows, const QModelIndex &parent)
{
    bool success = false;
    DomItem *parentItem = getItem(parent);

    beginRemoveRows(parent, position, position + rows - 1);
    success = parentItem->removeChildrens(position, rows);
    endRemoveRows();
    return success;
}
